import "Simbo.Babble.StyledList";

PatternList = class(StyledList);

local Constants =
{
	Border = 1,
	ScrollBarSize = 10,
	LineHeight = 20,
	CheckBoxWidth = 50,
	Colours =
	{
		NormalOutline = Turbine.UI.Color(1.0, 0.8, 0.65, 0.1),
		HoverOutline = Turbine.UI.Color(0.8, 0.65, 0.1),
		SelectedItem = Turbine.UI.Color(0.90, 0.25, 0.25, 0.25)
	}
}

function PatternList:Constructor(parent, checkboxes)
	StyledList.Constructor(self, parent);

	self.CheckBoxes = checkboxes;
	self.CheckBoxWidth = Constants.CheckBoxWidth;
	self.LabelWidth = Constants.CheckBoxWidth;
	
	self.SelectionListBox:SetMaxItemsPerLine(self.CheckBoxes + 1);
	
	self.CheckBoxSelected = function() end;
end

function PatternList:SetSize(width, height, checkBoxWidth)
	self.SelectionList:SetSize(width, height);
	self.SelectionListBox:SetSize(self.SelectionList:GetWidth() - (Constants.Border * 2), self.SelectionList:GetHeight() - (Constants.Border * 2));
	
	if checkBoxWidth ~= nil then
		self.CheckBoxWidth = checkBoxWidth;
	end
	self.LabelWidth = self.SelectionListBox:GetWidth() - self.CheckBoxWidth * self.CheckBoxes - Constants.ScrollBarSize;

	for i = 1, self.SelectionListBox:GetItemCount() do
		local item = self.SelectionListBox:GetItem(i);
		if i % (self.CheckBoxes + 1) == 1 then
			item:SetWidth(self.LabelWidth);
		else
			item:SetWidth(self.CheckBoxWidth);
		end
	end
	
	self.LinesScrollBar:SetSize(Constants.ScrollBarSize, self.SelectionListBox:GetHeight());
	self.LinesScrollBar:SetPosition(self.SelectionListBox:GetWidth() - Constants.ScrollBarSize,  0);
end

function PatternList:GetLabelWidth()
	return self.LabelWidth;
end

function PatternList:SetWidth(width)
	self:SetSize(width, self.SelectionList:GetHeight(), self.CheckBoxWidth);
end

function PatternList:SetHeight(height)
	self:SetSize(self.SelectionList:GetWidth(), height, self.CheckBoxWidth);
end
	
function PatternList:AddItem(text, selectedArray, selectNewItem)
	local insertIndex = self:FindInsertIndex(text)
	if insertIndex == nil then
		return;
	end
	
	local label = self:CreateLabelForList(text);
	label:SetWidth(self.LabelWidth);
	label.MouseEnter = function(sender, args)
		self:HighlightLabel(label);
	end
	label.MouseLeave = function(sender, args)
		self:UnhighlightLabel(label);
	end
	
	self.SelectionListBox:InsertItem(insertIndex, label);
	for index = 1, self.CheckBoxes do
		local checkbox = self:CreateCheckBoxForList(label, selectedArray[tostring(index)], index);
		self.SelectionListBox:InsertItem(insertIndex + index, checkbox);
	end

	if selectNewItem then
		self:SelectLine(insertIndex);
	else
		if self.SelectedLineIndex ~= nil and self.SelectedLineIndex >= insertIndex then
			self.SelectedLineIndex = self.SelectedLineIndex + self.SelectionListBox:GetMaxItemsPerLine();
		end
	end
end

function PatternList:CreateCheckBoxForList(label, selected, index)
	local checkbox = Turbine.UI.Lotro.CheckBox();
	checkbox:SetParent(self.SelectionListBox);
	checkbox:SetSize(self.CheckBoxWidth, Constants.LineHeight);
	checkbox:SetText("");
	checkbox:SetChecked(selected);
	checkbox.MouseEnter = function(sender, args)
		self:HighlightLabel(label);
	end
	checkbox.MouseLeave = function(sender, args)
		self:UnhighlightLabel(label);
	end
	checkbox.CheckedChanged = function(sender, args)
		local allSelected = self:IsAllSelected(index);
		self.CheckBoxSelected(self, { Index = index, AllSelected = allSelected });
	end
	
	return checkbox;
end

function PatternList:GetSelectedItem()
	if self.SelectedLineIndex > 0 then
		local item = self.SelectionListBox:GetItem(self.SelectedLineIndex);
		local itemToReturn = 
		{
			Text = item:GetText();
			CheckBoxes = {}
		};
		for itemIndex = 1, self.CheckBoxes do
			local item = self.SelectionListBox:GetItem(self.SelectedLineIndex + itemIndex);
			itemToReturn.CheckBoxes[tostring(itemIndex)] = item:IsChecked();
		end
		return itemToReturn;
	end
	
	return nil;
end

function PatternList:GetAllItems(selectedOnly)
	local itemCount = self.SelectionListBox:GetItemCount();
	local itemsPerLine = self.SelectionListBox:GetMaxItemsPerLine();
	local itemsToReturn = {};
	local returnIndex = 1;
	
	for index = 1, itemCount, itemsPerLine do
		local item = self.SelectionListBox:GetItem(index);
		local itemToReturn = 
		{
			Text = item:GetText();
			CheckBoxes = {}
		};
		for itemIndex = 1, itemsPerLine - 1 do
			local item = self.SelectionListBox:GetItem(index + itemIndex);
			itemToReturn.CheckBoxes[tostring(itemIndex)] = item:IsChecked();
		end
		itemsToReturn[tostring(returnIndex)] = itemToReturn;
		
		returnIndex = returnIndex + 1;
	end
	
	return itemsToReturn;
end

function PatternList:SetAllCheckboxes(checkboxIndex, checked)
	local itemCount = self.SelectionListBox:GetItemCount();
	local itemsPerLine = self.SelectionListBox:GetMaxItemsPerLine();

	for index = 1, itemCount, itemsPerLine do
		local item = self.SelectionListBox:GetItem(index + checkboxIndex);
		item:SetChecked(checked);
	end
end

function PatternList:IsAllSelected(checkboxIndex)
	local itemCount = self.SelectionListBox:GetItemCount();
	local itemsPerLine = self.SelectionListBox:GetMaxItemsPerLine();

	local allSelected = true;
	
	for index = 1, itemCount, itemsPerLine do
		local item = self.SelectionListBox:GetItem(index + checkboxIndex);
		if not item:IsChecked() then
			allSelected = false;
			break;
		end
	end
	
	return allSelected;
end